from odk_mailer.lib import utils
from types import SimpleNamespace

class Message:
    sender: str
    subject: str
    source: str
    content: str
    format: str

    def __init__(self, data) -> None:

        if type(data) is dict:
            self.sender = data["sender"]
            self.subject = data["subject"]
            self.source = data["source"]
            self.format = data["type"]
            
            if self.source == "stdin":
                self.content = data["location"]

            if self.source == "path":            
                with open(data["location"]) as f:
                    data_string = f.read()
                    base64_string = utils.base64_encode_str(data_string)
                    self.content = base64_string

            if self.source == "url":
                #tbd: read from url and store as base64
                self.content = data["location"]

        if type(data) is SimpleNamespace:
            self.sender = data.sender
            self.subject = data.subject
            self.source = data.source
            self.format = data.format
            self.content = data.content


from odk_mailer.classes.Config import Config
import smtplib

def send(email, verbose=False, config=[]):
    
    if not config:

        odk_mailer_config = Config()
        if not odk_mailer_config:
            raise Exception("Fatal Error: No Configuration File found.")

        smtp_host = odk_mailer_config.smtp_host
        smtp_port = odk_mailer_config.smtp_port
        smtp_user = odk_mailer_config.smtp_user
        smtp_pass = odk_mailer_config.smtp_pass
    else: 
        smtp_host, smtp_port, smtp_user, smtp_pass = config

    try:            
        smtp = smtplib.SMTP(timeout=5)
        if verbose:
        # enable debugging by CLI flag --debug
            smtp.set_debuglevel(2)

        smtp.connect(smtp_host, smtp_port)

        if smtp_user and smtp_pass:
        # if username and password are supplied, smtp.login()            
            smtp.login(smtp_user, smtp_pass)
        
        smtp.send_message(email)
        smtp.quit()

        if verbose:
            print()
            print("Successfully sent email to " + email["To"])
            print()
        # write into /log/timestamp_<hash>.log
        # tbd: log.write("Successfully sent email to " + email["To"])
        return True
    except Exception as error:

        if verbose:
            # write into /log/timestamp_<hash>.log
            # raise exception to interrupt loop
            print(error)
            print("Failed sending mail to: " + email['To'])
            print()
        
        return False


from odk_mailer.classes import Mailjob
from odk_mailer.lib import utils, smtp

from email.message import EmailMessage


class Mailer:
    mailjob: Mailjob
    verbose: bool
    dry: bool

    def __init__(self, dry=False, verbose=False) -> None:
        self.dry = dry
        self.verbose = verbose

    def send(self, mailjob: Mailjob):

        self.mailjob = mailjob
        errors=[]

        if self.verbose:
            print("ok")
            print()
            print(f"Run {self.mailjob.hash}")
            print("=====================================================================")
            if self.dry:
                print("Dry Run enabled -no Emails will be sent.")
            else:
                print("Sending emails..")
            print("=====================================================================")


        # Decode base64 if needed
        if self.mailjob.message.source == "stdin":
            text = self.mailjob.message.content
        if self.mailjob.message.source in ["path", "url"]:
            text = utils.base64_decode_str(self.mailjob.message.content)

        if self.dry:
            print()
            print("Message Summary")
            print("-----------------------------------------")
            print(f"sender: {self.mailjob.message.sender}")
            print(f"subject: {self.mailjob.message.subject}")
            print(f"source: {self.mailjob.message.source}")
            print(f"format: {self.mailjob.message.format}")
            print(f"# recipients: {len(self.mailjob.recipients)}")
            print("-----------------------------------------")
            print()

            for idx, recipient in enumerate(self.mailjob.recipients):
                content = utils.safe_format_map(text, recipient)
                print(f"#{idx} ({recipient.email})")
                print()
                print(content)

        else:            
            for idx, recipient in enumerate(self.mailjob.recipients):

                content = utils.safe_format_map(text, recipient)

                email = EmailMessage()
                email['From'] = self.mailjob.message.sender
                email['To'] = recipient.email
                email['Subject'] = self.mailjob.message.subject
                email.set_content(content, subtype=self.mailjob.message.format)

                if self.verbose:
                    print()
                    print(f"(#{idx+1}) Attempting to send.. ")

                sent = smtp.send(email, self.verbose, [])

                if sent:
                    if self.verbose:
                        print("Mailer Success!")

                if not sent:
                    errors.append(recipient)
                    if self.verbose:
                        print("Mailer Failure!")

        return [ len(errors) > 0, errors]